unit CollectionClientMainForm;

interface

uses
  Collection_TLB, ActiveX,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls;

type
  TCollectionClientForm = class(TForm)
    lstEditText: TListBox;
    btnNumericIndex: TButton;
    btnStringIndex: TButton;
    btnClearList: TButton;
    btnEnumerate: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnClearListClick(Sender: TObject);
    procedure btnNumericIndexClick(Sender: TObject);
    procedure btnStringIndexClick(Sender: TObject);
    procedure btnEnumerateClick(Sender: TObject);
  private
    { Private declarations }
  public
    EditControls: IEditControls;
    procedure EnumerationCallBack(Item: IDispatch; var Continue: Boolean);
  end;

var
  CollectionClientForm: TCollectionClientForm;

implementation

{$R *.DFM}

uses
  ComObj;

procedure TCollectionClientForm.FormCreate(Sender: TObject);
var
  Server: IApplication;
begin
  Server := CreateOleObject('Collection.Application') as IApplication;
  EditControls := Server.EditControls;
end;

procedure TCollectionClientForm.btnClearListClick(Sender: TObject);
begin
  lstEditText.Items.Clear;
end;

procedure TCollectionClientForm.btnNumericIndexClick(Sender: TObject);
var
  I: Integer;
  Edit: IEditControl;
begin
  for I := 1 to 5 do
  begin
    //Item is the default member of the collection object, so is optional
    //Edit := EditControls.Item[I];
    Edit := EditControls[I];
    lstEditText.Items.Add(Edit.Text);
    Edit.Text := Edit.Text + '!'
  end
end;

procedure TCollectionClientForm.btnStringIndexClick(Sender: TObject);
var
  I: Integer;
  EditName: String;
  Edit: IEditControl;
begin
  for I := 1 to 5 do
  begin
    EditName := Format('Edit%d', [I]);
    Edit := EditControls[EditName];
    lstEditText.Items.Add(Edit.Text);
    Edit.Text := Edit.Text + '!'
  end
end;

type
  TEnumerateCallBack = procedure(Item: IDispatch; var Continue: Boolean) of object;

procedure Enumerate(Collection: IDispatch; CallBack: TEnumerateCallBack);
var
  EnumUnk, Element: OleVariant;
  Enum: IEnumVariant;
  Fetched: Cardinal;
  Continue: Boolean;
const
  DispParamNoArgs: TDispParams =
    (rgvarg: nil; rgdispidNamedArgs: nil; cArgs: 0; cNamedArgs: 0);
begin
  //Call _NewEnum via its DispID and get enumerator object's IUnknown interface
  OleCheck(Collection.Invoke(DISPID_NEWENUM, GUID_NULL, 0,
    DISPATCH_PROPERTYGET, DispParamNoArgs, @EnumUnk, nil, nil));
  //Check we got an IUnknown and get IEnumVariant from it
  if VarType(EnumUnk) = varUnknown then
  begin
    Enum := IUnknown(EnumUnk) as IEnumVariant;
    Continue := True;
    //Use enumeration method to cycle through elements
    while (Enum.Next(1, Element, Fetched) = S_OK) and Continue do
      CallBack(Element, Continue)
  end
end;

//Custom method in form class that acts as a callback routine
procedure TCollectionClientForm.EnumerationCallBack(Item: IDispatch; var Continue: Boolean);
var
  Edit: IEditControl;
begin
  Edit := Item as IEditControl;
  lstEditText.Items.Add(Edit.Text);
  Edit.Text := Edit.Text + '!'
end;

//Initiating the enumeration with a button
procedure TCollectionClientForm.btnEnumerateClick(Sender: TObject);
begin
  Enumerate(EditControls as IDispatch, EnumerationCallBack)
end;

end.
