unit EditControlsImpl;

interface

uses
  ComObj, ActiveX, Collection_TLB, StdVcl, Classes, SysUtils, Windows;

type
  TEditControls = class(TAutoIntfObject, IEditControls)
  private
    FApplication: IApplication;
    FEditList: TInterfaceList;
  protected
    function Get__NewEnum: IUnknown; safecall;
    function Get_Application: IApplication; safecall;
    function Get_Count: Integer; safecall;
    function Get_Item(Index: OleVariant): IEditControl; safecall;
    { Protected declarations }
  public
    constructor Create(Application: IApplication);
    destructor Destroy; override;
  end;

  IEnumVariant = interface(IUnknown)
    ['{00020404-0000-0000-C000-000000000046}']
    function Next(celt: LongWord; out elt;
      pceltFetched: PLongWord): HResult; stdcall;
    function Skip(celt: LongWord): HResult; stdcall;
    function Reset: HResult; stdcall;
    function Clone(out Enum: IEnumVariant): HResult; stdcall;
  end;

  TEnumerator = class(TInterfacedObject, IEnumVariant)
  private
    FEnumPos: Integer;
    FEditControls: IEditControls;
  protected
    function Next(celt: LongWord; out elt;
      pceltFetched: PLongWord): HResult; stdcall;
    function Skip(celt: LongWord): HResult; stdcall;
    function Reset: HResult; stdcall;
    function Clone(out Enum: IEnumVariant): HResult; stdcall;
  public
    constructor Create(EditControls: IEditControls);
  end;

implementation

uses
  ComServ, EditControlImpl, Forms, StdCtrls;

constructor TEditControls.Create(Application: IApplication);
var
  I: Integer;
begin
  inherited Create(ComServer.TypeLib, IEditControls);
  FApplication := Application;
  FEditList := TInterfaceList.Create;
  //Qualify the Forms unit, as Application is ambiguous in this unit
  with Forms.Application.MainForm do
    for I := 0 to ComponentCount - 1 do
      if Components[I] is TEdit then
        FEditList.Add(TEditControl.Create(FApplication, TEdit(Components[I])))
end;

destructor TEditControls.Destroy;
begin
  FEditList.Free;
  inherited;
end;

function TEditControls.Get__NewEnum: IUnknown;
begin
  Result := TEnumerator.Create(Self)
end;

function TEditControls.Get_Application: IApplication;
begin
  Result := FApplication
end;

function TEditControls.Get_Count: Integer;
begin
  Result := FEditList.Count
end;

function TEditControls.Get_Item(Index: OleVariant): IEditControl;
var
  I: Integer;
begin
  Result := nil;
  if VarType(Index) = varOleStr then
  begin
    //find by name
    for I := 0 to FEditList.Count - 1 do
      if CompareText((FEditList[I] as IEditControl).Name, Index) = 0 then
      begin
        Result := FEditList[I] as IEditControl;
        Break
      end
  end
  else
  try
    //Assume an integer was passed, and cater for collection starting at 1
    Result := FEditList[Index - 1] as IEditControl
  except
    //Bad index
  end
end;

{ TEnumerator }

constructor TEnumerator.Create(EditControls: IEditControls);
begin
  inherited Create;
  FEditControls := EditControls
end;

function TEnumerator.Clone(out Enum: IEnumVariant): HResult;
var
  Enumerator: TEnumerator;
begin
  Result := S_OK;
  try
    Enumerator := TEnumerator.Create(FEditControls);
    Enumerator.FEnumPos := FEnumPos;
    Enum := Enumerator;
  except
    Enum := nil;
    Result := E_OUTOFMEMORY
  end
end;

function TEnumerator.Next(celt: LongWord; out elt;
  pceltFetched: PLongWord): HResult;
var
  I: Integer;
  EditDispatch: IDispatch;
begin
  Result := S_FALSE;
  try
    if Assigned(pceltFetched) then
      pcEltFetched^ := 0;
    for I := 0 to celt - 1 do
    begin
      if FEnumPos >= FEditControls.Count then
        Exit;
      EditDispatch := FEditControls.Item[FEnumPos + 1] as IDispatch;
      //Interface reference is about to be stored in a pointer field.
      //Must therefore do lifetime management manually, as Delphi will not do it
      EditDispatch._AddRef;
      TVariantArgList(elt)[I].vt := varDispatch;
      TVariantArgList(elt)[I].dispVal := Pointer(EditDispatch);

      Inc(FEnumPos);
      if Assigned(pceltFetched) then
        Inc(pceltFetched^)
    end
  except
    //Hide any errors
  end;
  if (not Assigned(pceltFetched)) or (Assigned(pceltFetched) and (pceltFetched^ = celt)) then
    Result := S_OK
end;

function TEnumerator.Reset: HResult;
begin
  FEnumPos := 0;
  Result := S_OK
end;

function TEnumerator.Skip(celt: LongWord): HResult;
begin
  if FEnumPos + Integer(celt) > FEditControls.Count - 1 then
  begin
    FEnumPos := FEditControls.Count - 1;
    Result := S_FALSE;
  end
  else
  begin
    Inc(FEnumPos, celt);
    Result := S_OK
  end
end;

end.
